/*
 *  Copyright (C) 2007  Thadeu Lima de Souza Cascardo <cascardo@minaslivre.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <villa.h>
#include <cabin.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include "avaliacao.h"

VILLA *db;

void
gtk_container_remove_r (GtkWidget *widget, gpointer data)
{
  gtk_container_remove (data, widget);
}

void
gtk_container_remove_all (GtkContainer *container)
{
  gtk_container_foreach (container, gtk_container_remove_r, container);
}


void
addcur (const gchar *key, const gchar *val)
{
  char *smap;
  int slen;
  CBMAP *map;
  smap = vlcurval (db, &slen);
  map = cbmapload (smap, slen);
  cbmapput (map, key, -1, val, -1, 1);
  free (smap);
  smap = cbmapdump (map, &slen);
  cbmapclose (map);
  vlcurput (db, smap, slen, VL_CPCURRENT);
}

void
showcur (gpointer data)
{
  char *val;
  int vlen;
  CBMAP *map;
  const char *mapkey;
  GtkWidget *hbox;
  GtkWidget *label;
  GtkWidget *entry;
  GtkSizeGroup *sgroup;
  sgroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
  gtk_container_remove_all (GTK_CONTAINER (data));
  val = vlcurval (db, &vlen);
  map = cbmapload (val, vlen);
  cbmapiterinit (map);
  for (mapkey = cbmapiternext (map, NULL); mapkey != NULL;
       mapkey = cbmapiternext (map, NULL))
    {
      const char *mapval;
      mapval = cbmapget (map, mapkey, -1, NULL);
      hbox = gtk_hbox_new (FALSE, 5);
      gtk_box_pack_start (GTK_BOX (data), hbox, FALSE, FALSE, 0);
      label = gtk_label_new (mapkey);
      g_object_set (G_OBJECT (label), "xalign", 0.0, NULL);
      gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
      entry = gtk_entry_new ();
      gtk_entry_set_text (GTK_ENTRY (entry), mapval);
      gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
      gtk_size_group_add_widget (sgroup, label);
    }
  cbmapclose (map);
  g_object_unref (sgroup);
  gtk_widget_show_all (GTK_WIDGET (data));
  free (val);
}

void
first (GtkButton *button, gpointer data)
{
  if (vlcurfirst (db))
    showcur (data);
}

void
next (GtkButton *button, gpointer data)
{
  if (vlcurnext (db))
    showcur (data);
}

void
prev (GtkButton *button, gpointer data)
{
  if (vlcurprev (db))
    showcur (data);
}

void
last (GtkButton *button, gpointer data)
{
  if (vlcurlast (db))
    showcur (data);
}

void
removecur (GtkButton *button, gpointer data)
{
  if (vlcurout (db))
    showcur (data);
}

void
addfield (GtkButton *button, gpointer data)
{
  GList *children;
  GtkEntry *key;
  GtkEntry *value;
  children = gtk_container_get_children (GTK_CONTAINER (data));
  key = children->data;
  value = children->next->data;
  g_list_free (children);
  addcur (gtk_entry_get_text (key), gtk_entry_get_text (value));
}

void
viewcur (GtkButton *button, gpointer data)
{
  GtkTextBuffer *buffer;
  gchar *message;
  message = regex_create_message ();
  buffer = gtk_text_buffer_new (NULL);
  gtk_text_buffer_set_text (buffer, message, -1);
  g_free (message);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW (data), buffer);
}

void
ui (gpointer data)
{


  GtkWidget *window;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *button;
  GtkWidget *form;

  GtkWidget *hbox2;
  GtkWidget *entry;
  GtkWidget *button2;

  GtkWidget *text;

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_container_set_border_width (GTK_CONTAINER (window), 10);
  g_signal_connect (G_OBJECT (window), "destroy", gtk_main_quit, NULL);



  vbox = gtk_vbox_new (FALSE, 5);
  gtk_container_add (GTK_CONTAINER (window), vbox);

  hbox = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

  form = gtk_vbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox), form, FALSE, FALSE, 0);


  button = gtk_button_new_with_label ("First");
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK(first), form);
  button = gtk_button_new_with_label ("Prev");
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK(prev), form);
  button = gtk_button_new_with_label ("Next");
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK(next), form);
  button = gtk_button_new_with_label ("Last");
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK(last), form);


  hbox2 = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);

  entry = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (hbox2), entry, FALSE, FALSE, 0);
  entry = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (hbox2), entry, FALSE, FALSE, 0);
  button2 = gtk_button_new_with_label ("Add");
  gtk_box_pack_start (GTK_BOX (hbox2), button2, FALSE, FALSE, 0);
  g_signal_connect (G_OBJECT (button2), "clicked", G_CALLBACK (addfield), hbox2);

  button2 = gtk_button_new_with_label ("Remove");
  gtk_box_pack_start (GTK_BOX (hbox), button2, FALSE, FALSE, 0);
  g_signal_connect (G_OBJECT (button2), "clicked", G_CALLBACK (removecur), form);

  text = gtk_text_view_new ();
  gtk_box_pack_start (GTK_BOX (vbox), text, TRUE, TRUE, 0);

  button2 = gtk_button_new_with_label ("View");
  gtk_box_pack_start (GTK_BOX (hbox), button2, FALSE, FALSE, 0);
  g_signal_connect (G_OBJECT (button2), "clicked", G_CALLBACK (viewcur), text);

  gtk_widget_show_all (window);


}

int main (int argc, char **argv)
{
  gtk_init (&argc, &argv);
  regex_init ();
  db = vlopen ("palestras.vl", VL_OWRITER, VL_CMPINT);
  ui (db);
  gtk_main ();
  vlclose (db);
  return 0;
}
